# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from rlbot_action_server.models.action_choice import ActionChoice  # noqa: E501
from rlbot_action_server.models.api_response import ApiResponse  # noqa: E501
from rlbot_action_server.models.available_actions import AvailableActions  # noqa: E501
from rlbot_action_server.test import BaseTestCase


class TestActionController(BaseTestCase):
    """ActionController integration test stubs"""

    def test_choose_action(self):
        """Test case for choose_action

        
        """
        body = ActionChoice()
        response = self.client.open(
            '/action/choose',
            method='POST',
            data=json.dumps(body),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_actions_currently_available(self):
        """Test case for get_actions_currently_available

        
        """
        response = self.client.open(
            '/action/currentlyAvailable',
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



import connexion
from rlbot_action_server import bot_holder
from rlbot_action_server.models.action_choice import ActionChoice  # noqa: E501
from rlbot_action_server.models.api_response import ApiResponse  # noqa: E501


def choose_action(body):
    if connexion.request.is_json:
        body = ActionChoice.from_dict(connexion.request.get_json())
        action_broker = bot_holder.get_bot_action_broker()
        if action_broker is not None:
            response = action_broker.set_action(body)
            if response:
                return response
            return ApiResponse(200, f'Probably set action: {body.action.description}')


def get_actions_currently_available():
    action_broker = bot_holder.get_bot_action_broker()
    if action_broker is not None:
        return action_broker.get_actions_currently_available()



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from rlbot_action_server.models.base_model_ import Model
from rlbot_action_server.models.strategic_category import StrategicCategory  # noqa: F401,E501
from rlbot_action_server import util


class BotAction(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, description: str=None, action_type: str=None, strategic_category: StrategicCategory=None, data: Dict=None):  # noqa: E501
        """BotAction - a model defined in Swagger

        :param description: The description of this BotAction.  # noqa: E501
        :type description: str
        :param action_type: The action_type of this BotAction.  # noqa: E501
        :type action_type: str
        :param strategic_category: The strategic_category of this BotAction.  # noqa: E501
        :type strategic_category: StrategicCategory
        :param data: The data of this BotAction.  # noqa: E501
        :type data: Dict
        """
        self.swagger_types = {
            'description': str,
            'action_type': str,
            'strategic_category': StrategicCategory,
            'data': Dict
        }

        self.attribute_map = {
            'description': 'description',
            'action_type': 'actionType',
            'strategic_category': 'strategicCategory',
            'data': 'data'
        }
        self._description = description
        self._action_type = action_type
        self._strategic_category = strategic_category
        self._data = data

    @classmethod
    def from_dict(cls, dikt) -> 'BotAction':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The BotAction of this BotAction.  # noqa: E501
        :rtype: BotAction
        """
        return util.deserialize_model(dikt, cls)

    @property
    def description(self) -> str:
        """Gets the description of this BotAction.


        :return: The description of this BotAction.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description: str):
        """Sets the description of this BotAction.


        :param description: The description of this BotAction.
        :type description: str
        """

        self._description = description

    @property
    def action_type(self) -> str:
        """Gets the action_type of this BotAction.


        :return: The action_type of this BotAction.
        :rtype: str
        """
        return self._action_type

    @action_type.setter
    def action_type(self, action_type: str):
        """Sets the action_type of this BotAction.


        :param action_type: The action_type of this BotAction.
        :type action_type: str
        """
        if action_type is None:
            raise ValueError("Invalid value for `action_type`, must not be `None`")  # noqa: E501

        self._action_type = action_type

    @property
    def strategic_category(self) -> StrategicCategory:
        """Gets the strategic_category of this BotAction.


        :return: The strategic_category of this BotAction.
        :rtype: StrategicCategory
        """
        return self._strategic_category

    @strategic_category.setter
    def strategic_category(self, strategic_category: StrategicCategory):
        """Sets the strategic_category of this BotAction.


        :param strategic_category: The strategic_category of this BotAction.
        :type strategic_category: StrategicCategory
        """

        self._strategic_category = strategic_category

    @property
    def data(self) -> Dict:
        """Gets the data of this BotAction.


        :return: The data of this BotAction.
        :rtype: Dict
        """
        return self._data

    @data.setter
    def data(self, data: Dict):
        """Sets the data of this BotAction.


        :param data: The data of this BotAction.
        :type data: Dict
        """

        self._data = data



from rlbot_action_server.bot_action_broker import BotActionBroker


print('initializing _broker as None')
_broker: BotActionBroker = None

def set_bot_action_broker(broker: BotActionBroker):
    global _broker
    print(f'setting broker to {broker}')
    _broker = broker

def get_bot_action_broker() -> BotActionBroker:
    return _broker



import socket
from pathlib import Path
from typing import List

import connexion
from rlbot_action_server import encoder
from rlbot_action_server.models import AvailableActions, ActionChoice, ApiResponse


class BotActionBroker:
    def get_actions_currently_available(self) -> List[AvailableActions]:
        raise NotImplementedError()

    def set_action(self, action: ActionChoice) -> ApiResponse:
        """
        Returns true if the action was able to be applied successfully,
        false if it was rejected for some reason.
        """
        raise NotImplementedError()


def run_action_server(port: int):
    spec_dir = Path(__file__).parent / 'swagger'
    app = connexion.App(__name__, specification_dir=spec_dir)
    app.app.json_encoder = encoder.JSONEncoder
    app.add_api('swagger.yaml', arguments={'title': 'BotActionServer'}, pythonic_params=True)
    app.run(port=port)


def find_usable_port(desired_port: int):
    for port_to_test in range(desired_port, 65535):
        if is_port_accessible(port_to_test):
            return port_to_test
    raise PermissionError(
        'Unable to find a usable port for running an action server! Is your antivirus messing you up? '
        'Check https://github.com/RLBot/RLBot/wiki/Antivirus-Notes')


def is_port_accessible(port):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        try:
            sock.bind(('127.0.0.1', port))
            return True
        except:
            return False



# coding: utf-8

# flake8: noqa
from __future__ import absolute_import
# import models into model package
from rlbot_action_server.models.action_choice import ActionChoice
from rlbot_action_server.models.api_response import ApiResponse
from rlbot_action_server.models.available_actions import AvailableActions
from rlbot_action_server.models.bot_action import BotAction
from rlbot_action_server.models.strategic_category import StrategicCategory



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from rlbot_action_server.models.base_model_ import Model
from rlbot_action_server.models.bot_action import BotAction  # noqa: F401,E501
from rlbot_action_server import util


class ActionChoice(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, action: BotAction=None):  # noqa: E501
        """ActionChoice - a model defined in Swagger

        :param action: The action of this ActionChoice.  # noqa: E501
        :type action: BotAction
        """
        self.swagger_types = {
            'action': BotAction
        }

        self.attribute_map = {
            'action': 'action'
        }
        self._action = action

    @classmethod
    def from_dict(cls, dikt) -> 'ActionChoice':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ActionChoice of this ActionChoice.  # noqa: E501
        :rtype: ActionChoice
        """
        return util.deserialize_model(dikt, cls)

    @property
    def action(self) -> BotAction:
        """Gets the action of this ActionChoice.


        :return: The action of this ActionChoice.
        :rtype: BotAction
        """
        return self._action

    @action.setter
    def action(self, action: BotAction):
        """Sets the action of this ActionChoice.


        :param action: The action of this ActionChoice.
        :type action: BotAction
        """

        self._action = action



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from rlbot_action_server.models.base_model_ import Model
from rlbot_action_server import util


class StrategicCategory(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    allowed enum values
    """
    NEUTRAL = "neutral"
    OFFENSE = "offense"
    DEFENSE = "defense"
    def __init__(self):  # noqa: E501
        """StrategicCategory - a model defined in Swagger

        """
        self.swagger_types = {
        }

        self.attribute_map = {
        }

    @classmethod
    def from_dict(cls, dikt) -> 'StrategicCategory':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The StrategicCategory of this StrategicCategory.  # noqa: E501
        :rtype: StrategicCategory
        """
        return util.deserialize_model(dikt, cls)



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from rlbot_action_server.models.base_model_ import Model
from rlbot_action_server import util


class ApiResponse(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, code: int=None, message: str=None):  # noqa: E501
        """ApiResponse - a model defined in Swagger

        :param code: The code of this ApiResponse.  # noqa: E501
        :type code: int
        :param message: The message of this ApiResponse.  # noqa: E501
        :type message: str
        """
        self.swagger_types = {
            'code': int,
            'message': str
        }

        self.attribute_map = {
            'code': 'code',
            'message': 'message'
        }
        self._code = code
        self._message = message

    @classmethod
    def from_dict(cls, dikt) -> 'ApiResponse':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ApiResponse of this ApiResponse.  # noqa: E501
        :rtype: ApiResponse
        """
        return util.deserialize_model(dikt, cls)

    @property
    def code(self) -> int:
        """Gets the code of this ApiResponse.


        :return: The code of this ApiResponse.
        :rtype: int
        """
        return self._code

    @code.setter
    def code(self, code: int):
        """Sets the code of this ApiResponse.


        :param code: The code of this ApiResponse.
        :type code: int
        """

        self._code = code

    @property
    def message(self) -> str:
        """Gets the message of this ApiResponse.


        :return: The message of this ApiResponse.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: str):
        """Sets the message of this ApiResponse.


        :param message: The message of this ApiResponse.
        :type message: str
        """

        self._message = message



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from rlbot_action_server.models.base_model_ import Model
from rlbot_action_server.models.bot_action import BotAction  # noqa: F401,E501
from rlbot_action_server import util


class AvailableActions(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, entity_name: str=None, current_action: BotAction=None, available_actions: List[BotAction]=None):  # noqa: E501
        """AvailableActions - a model defined in Swagger

        :param entity_name: The entity_name of this AvailableActions.  # noqa: E501
        :type entity_name: str
        :param current_action: The current_action of this AvailableActions.  # noqa: E501
        :type current_action: BotAction
        :param available_actions: The available_actions of this AvailableActions.  # noqa: E501
        :type available_actions: List[BotAction]
        """
        self.swagger_types = {
            'entity_name': str,
            'current_action': BotAction,
            'available_actions': List[BotAction]
        }

        self.attribute_map = {
            'entity_name': 'entityName',
            'current_action': 'currentAction',
            'available_actions': 'availableActions'
        }
        self._entity_name = entity_name
        self._current_action = current_action
        self._available_actions = available_actions

    @classmethod
    def from_dict(cls, dikt) -> 'AvailableActions':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The AvailableActions of this AvailableActions.  # noqa: E501
        :rtype: AvailableActions
        """
        return util.deserialize_model(dikt, cls)

    @property
    def entity_name(self) -> str:
        """Gets the entity_name of this AvailableActions.

        The name of the bot or script that these actions are associated with.  # noqa: E501

        :return: The entity_name of this AvailableActions.
        :rtype: str
        """
        return self._entity_name

    @entity_name.setter
    def entity_name(self, entity_name: str):
        """Sets the entity_name of this AvailableActions.

        The name of the bot or script that these actions are associated with.  # noqa: E501

        :param entity_name: The entity_name of this AvailableActions.
        :type entity_name: str
        """

        self._entity_name = entity_name

    @property
    def current_action(self) -> BotAction:
        """Gets the current_action of this AvailableActions.


        :return: The current_action of this AvailableActions.
        :rtype: BotAction
        """
        return self._current_action

    @current_action.setter
    def current_action(self, current_action: BotAction):
        """Sets the current_action of this AvailableActions.


        :param current_action: The current_action of this AvailableActions.
        :type current_action: BotAction
        """

        self._current_action = current_action

    @property
    def available_actions(self) -> List[BotAction]:
        """Gets the available_actions of this AvailableActions.


        :return: The available_actions of this AvailableActions.
        :rtype: List[BotAction]
        """
        return self._available_actions

    @available_actions.setter
    def available_actions(self, available_actions: List[BotAction]):
        """Sets the available_actions of this AvailableActions.


        :param available_actions: The available_actions of this AvailableActions.
        :type available_actions: List[BotAction]
        """

        self._available_actions = available_actions



# coding: utf-8

import sys
from setuptools import setup, find_packages

NAME = "rlbot_action_server"
VERSION = "1.0.0"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["connexion"]

setup(
    name=NAME,
    version=VERSION,
    description="Bot Action Server",
    author_email="rlbotofficial@gmail.com",
    url="",
    keywords=["Swagger", "Bot Action Server"],
    install_requires=REQUIRES,
    packages=find_packages(),
    package_data={'': ['swagger/swagger.yaml']},
    include_package_data=True,
    entry_points={
        'console_scripts': ['rlbot_action_server=rlbot_action_server.__main__:main']},
    long_description="""\
    Allows custom Rocket League bots to accept tactical suggestions in the middle of a game.
    """
)
